''
''	Example of how to use easyChipmunk
''

#include "../easyChipmunk.bi"
#include "fbgfx.bi"
using fb

' Change these values and recompile to change the simulation behaviour
#define SPAWN_BOUNCY	0.1
#define	SPAWN_FRICTION	0.01

#define GRABABLE_MASK_BIT (1 shl 31)
#define NOT_GRABABLE_MASK (not GRABABLE_MASK_BIT)

type shapeData
	shape					as cpShape ptr
	body					as cpBody ptr
	colour					as uinteger
end type

declare sub setupScreen( byval w as integer, byval h as integer, byval instance as easyChipmunk ptr )
declare function generateShapeData( list() as shapeData, byval shape as cpShape ptr, byval colour as uinteger ) as integer
declare function createNewCircle( list() as shapeData, byval mass as cpFloat, byval radius as cpFloat, byval colour as uinteger, byval isStatic as ubyte = 0, byval offset as cpVect = cpvzero ) as cpShape ptr
declare function createNewPolygon( list() as shapeData, byval mass as cpFloat, byval vertices as cpVect ptr, byval nVertices as integer, byval colour as uinteger, byval isStatic as ubyte = 0, byval offset as cpVect = cpvzero ) as cpShape ptr
declare sub removeShape( list() as shapeData, byval instance as easyChipmunk ptr, byval shape as cpShape ptr )
declare sub addShapeToSpace( list() as shapeData, byval instance as easyChipmunk ptr, byval shape as cpShape ptr )
declare sub drawShapeList( list() as shapeData )

dim physics as easyChipmunk = easyChipmunk()
redim shapes(0) as shapeData

physics.lock()
	setupScreen( 800, 600, @physics )
	physics.space->gravity = cpv( 0, 100 )
	physics.space->sleepTimeThreshold = 0.05
	physics.space->idleSpeedThreshold = 1.0
	physics.pause = 0
physics.unlock()

dim as integer mx, my, mb
dim as double spawnTimer = Timer
dim as cpConstraint ptr mouseSpring = NULL
dim as cpBody ptr mousePointer = cpBodyNew( INFINITY, INFINITY )
dim as integer frameTimer = Timer + 1
dim as integer framecount = 0, fps = 0, shapecount = 0

do
	physics.simulate()
	physics.lock()
		screenlock()
			cls
			'locate 1, 2: print "Left click to move around objects"
			'locate 2, 2: print "Right click to spawn a random object"
			'locate 3, 2: print "Right click + Ctrl to remove an object"
			'locate 4, 2 : Print Using "Simulating #### shapes"; shapecount
			locate 1, LoWord( width() ) - 9 : print using "##### fps"; fps
			drawShapeList( shapes() )
			if mouseSpring <> NULL then
				line( mouseSpring->a->p.x, mouseSpring->a->p.y )-( mouseSpring->b->p.x, mouseSpring->b->p.y ), rgb( 200, 200, 255 )
			end if
		screenunlock()
		getmouse mx, my, , mb
		if mb = -1 then mb = 0
		if ( mx <> -1 ) and ( my <> -1 ) then mousePointer->p = cpv( mx, my )
		if ( mb And 1 ) then
			if mouseSpring = NULL then
				var query = cpSpacePointQueryFirst( physics.space, mousePointer->p, GRABABLE_MASK_BIT, 0 )
				if query <> NULL then
					mouseSpring = cpDampedSpringNew( mousePointer, query->body, cpvzero, cpBodyWorld2Local( query->body, mousePointer->p ), 1.0, 100.0, 2.0 )
					mouseSpring->biasCoef = 1.0
					mouseSpring->maxForce = INFINITY
					cpSpaceAddConstraint( physics.space, mouseSpring )
				end if
			end if
		else
			if mouseSpring <> NULL then
				cpSpaceRemoveConstraint( physics.space, mouseSpring )
				cpConstraintFree( mouseSpring )
				mouseSpring = NULL
			end if
		end if
		if (mb And 2 ) then
			if multikey( SC_CONTROL ) then
				var query = cpSpacePointQueryFirst( physics.space, mousePointer->p, GRABABLE_MASK_BIT, 0 )
				if query <> NULL then
					removeShape( shapes(), @physics, query )
				end if
			else
				if ( Timer > spawnTimer ) And ( ( mousePointer->p.x > 0 ) And ( mousePointer->p.y > 0 ) And ( mousePointer->p.x < 800 ) And ( mousePointer->p.y < 600 ) ) then
					spawnTimer = Timer + 0.05
					var shape = createNewCircle( shapes(), 10, 10.0, rgb( 100 + int(rnd*150), 100 + int(rnd*150), 100 + int(rnd*150) ) )
					shape->e = SPAWN_BOUNCY
					shape->u = SPAWN_FRICTION
					shape->body->p = mousePointer->p
					addShapeToSpace( shapes(), @physics, shape )
				end if
			end if
			shapecount = ubound( shapes )
		end if
		if multikey( SC_C ) And lbound(shapes) > 0 then
			removeShape( shapes(), @physics, shapes(1).shape )
			shapecount = ubound( shapes )
		end if
	physics.unlock()
	
	framecount += 1
	if Timer > frameTimer then
		fps = framecount
		framecount = 0
		frameTimer = Timer + 1
	end if
loop until multikey( &h01 )

end 0

sub setupScreen( byval w as integer, byval h as integer, byval instance as easyChipmunk ptr )
	screenres w, h, 16, , GFX_ALPHA_PRIMITIVES Or GFX_NO_SWITCH
	Randomize Timer
	
	dim wallVerts(0 to 3) as cpVect = { cpv(0, 0), cpv(w,0), cpv(w,h), cpv(0, h) }
	dim j as integer
	for i as integer = 0 to 3
		j = i + 1
		if j > 3 then j = 0
		dim ws as cpShape ptr
		ws = cpSegmentShapeNew( NULL, wallVerts(i), wallVerts(j), 15.0 )
		ws->u = 0.1
		ws->e = 1.0
		ws->layers = NOT_GRABABLE_MASK
		cpSpaceAddStaticShape( instance->space, ws )
	next i
end sub

function generateShapeData( list() as shapeData, byval shape as cpShape ptr, byval colour as uinteger ) as integer
	dim idx as integer = -1
	for i as integer = 1 to ubound( list )
		if list(i).shape = NULL then
			idx = i
			exit for
		end if
	next
	if idx = -1 then
		idx = ubound( list ) + 1
		redim preserve list(1 to idx) as shapeData
	end if
	list(idx).shape = shape
	list(idx).body = NULL
	list(idx).colour = colour
	return idx
end function

function createNewCircle( list() as shapeData, byval mass as cpFloat, byval radius as cpFloat, byval colour as uinteger, byval isStatic as ubyte = 0, byval offset as cpVect = cpvzero ) as cpShape ptr
	dim shape as cpShape ptr
	dim body as cpBody ptr = NULL
	if isStatic = 0 then
		body = cpBodyNew( mass, cpMomentForCircle( mass, radius, 0.0, offset ) )
	end if
	shape = cpCircleShapeNew( body, radius, offset )
	var i = generateShapeData( list(), shape, colour )
	list(i).body = body
	return shape
end function

function createNewPolygon( list() as shapeData, byval mass as cpFloat, byval vertices as cpVect ptr, byval nVertices as integer, byval colour as uinteger, byval isStatic as ubyte = 0, byval offset as cpVect = cpvzero ) as cpShape ptr
	dim shape as cpShape ptr = NULL
	dim body as cpBody ptr = NULL
	if cpPolyValidate( vertices, nVertices ) = cpFalse then return NULL
	if isStatic = 0 then
		body = cpBodyNew( mass, cpMomentForPoly( mass, nVertices, vertices, offset ) )
	end if
	shape = cpPolyShapeNew( body, nVertices, vertices, offset )
	var i = generateShapeData( list(), shape, colour )
	list(i).body = body
	return shape
end function

sub removeShape( list() as shapeData, byval instance as easyChipmunk ptr, byval shape as cpShape ptr )
	for i as integer = 1 to ubound( list )
		if list(i).shape = shape then
			if list(i).body then cpSpaceRemoveBody( instance->space, list(i).body )
			cpSpaceRemoveShape( instance->space, list(i).shape )
			list(i).shape = NULL
			for r as integer = i to ubound(list)-1
				list(r) = list(r+1)
			next
			if ubound(list) > 1 then
				redim preserve list(1 to ubound(list)-1) as shapeData
			else
				redim list(0) as shapeData
			end if
			return
		end if
	next
end sub

sub addShapeToSpace( list() as shapeData, byval instance as easyChipmunk ptr, byval shape as cpShape ptr )
	for i as integer = 1 to ubound( list )
		if list(i).shape = shape then
			if list(i).body then cpSpaceAddBody( instance->space, list(i).body )
			cpSpaceAddShape( instance->space, list(i).shape )
			return
		end if
	next
end sub

sub drawShapeList( list() as shapeData )
	dim node as cpShape ptr
	for i as integer = 1 to ubound( list )
		node = list(i).shape
		if node = NULL then continue for
		select case node->klass->type
		case CP_CIRCLE_SHAPE
			circle( node->body->p.x, node->body->p.y ), cpCircleShapeGetRadius( node ), list(i).colour
			pset( node->body->p.x, node->body->p.y ), rgb(255,200,200)
		case CP_POLY_SHAPE
			var center = node->body->p
			var vert = cpPolyShapeGetVert( node, 0 )
			pset( center.x + vert.x, center.y + vert.y ), list(i).colour
			for i as integer = 1 to cpPolyShapeGetNumVerts( node )-1
				var vert = cpPolyShapeGetVert( node, i )
				line -( center.x + vert.x, center.y + vert.y ), list(i).colour
			next
		end select
	next
end sub

