/' Copyright (c) 2007 Scott Lembcke
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 '/

#ifndef CHIPMUNK_HEADER
#define CHIPMUNK_HEADER	1

declare sub cpMessage cdecl alias "cpMessage" ( byval as zstring ptr, byval as zstring ptr, byval as zstring ptr, byval as integer, byval as integer )
#ifdef NDEBUG
	#define	cpAssertWarn(condition, message)
#else
	#define cpAssertWarn(condition, message) if (not (condition)) then : cpMessage(message, #condition, __FILE__, __LINE__, 0) : end if
#endif

#ifdef NDEBUG
	#define	cpAssert(condition, message)
#else
	#define cpAssert(condition, message) if (not(condition)) then : cpMessage(message, #condition, __FILE__, __LINE__, 1) : end if
#endif

#include "chipmunk_types.bi"

#define INFINITY (1e31)

'' Maximum allocated size for various Chipmunk buffer sizes
#define CP_BUFFER_BYTES (32*1024)

#include "crt/stdlib.bi"	'' Get malloc() and friends..
#define cpmalloc malloc
#define cpcalloc calloc
#define cprealloc realloc
#define cpfree free

#ifndef NULL
#define NULL 0
#endif

#include "cpVect.bi"
#include "cpBB.bi"
#include "cpArray.bi"
#include "cpHashSet.bi"
#include "cpSpaceHash.bi"

#include "cpBody.bi"
#include "cpShape.bi"
#include "cpPolyShape.bi"

#include "cpArbiter.bi"
#include "cpCollision.bi"
	
#include "constraints/cpConstraint.bi"

#include "cpSpace.bi"

#define CP_HASH_COEF (3344921057ul)
#define CP_HASH_PAIR(A, B) ((cpHashValue)(A)*CP_HASH_COEF ^ (cpHashValue)(B)*CP_HASH_COEF)

extern cpVersionString alias "cpVersionString" as const zstring ptr
declare sub cpInitChipmunk cdecl alias "cpInitChipmunk" ()

'' [NOTE] I added this function for ease of use
function cpVersion() as string
	return str( *cpVersionString )
end function

'' Calculate the moment of inertia for a circle, r1 and r2 are the inner and outer diameters.
'' (A solid circle has an inner diameter of 0)
declare function cpMomentForCircle cdecl alias "cpMomentForCircle" ( byval as cpFloat, byval as cpFloat, byval as cpFloat, byval as cpVect ) as cpFloat

'' Calculate the moment of inertia for a line segment. (beveling radius not supported)
declare function cpMomentForSegment cdecl alias "cpMomentForSegment" ( byval as cpFloat, byval as cpVect, byval as cpVect ) as cpFloat

'' Calculate the moment of inertia for a solid polygon shape.
declare function cpMomentForPoly cdecl alias "cpMomentForPoly" ( byval as cpFloat, byval as integer, byval as cpVect ptr, byval as cpVect ) as cpFloat

'' Calculate the moment of inertia for a solid box.
declare function cpMomentForBox cdecl alias "cpMomentForBox" ( byval as cpFloat, byval as cpFloat, byval as cpFloat ) as cpFloat

#endif
