
#ifndef CP_USE_DOUBLES
  '' use doubles by default for higher precision
  #define CP_USE_DOUBLES 1
#endif

#if CP_USE_DOUBLES
	type cpFloat as double
#else
	type cpFloat as single
#endif

#define cpfsqrt sqr
#define cpfsin sin
#define cpfcos cos
#define cpfacos acos
#define cpfatan2 atan2
#define cpfmod fmod
#define cpfexp exp
#define cpfpow pow
#define cpffloor floor
#define cpfceil ceil

function cpfmax( a as cpFloat, b as cpFloat ) as cpFloat
	return iif( a > b, a, b )
end function

function cpfmin( a as cpFloat, b as cpFloat ) as cpFloat
	return iif( a < b, a, b )
end function

function cpfabs( n as cpFloat ) as cpFloat
	return iif( n < 0, -n, n )
end function

function cpfclamp( f as cpFloat, min as cpFloat, max as cpFloat ) as cpFloat
	return cpfmin( cpfmax( f, min ), max )
end function

function cpflerp( f1 as cpFloat, f2 as cpFloat, t as cpFloat ) as cpFloat
	return f1 * ( 1.0 - t ) + f2 * t
end function

function cpflerpconst( f1 as cpFloat, f2 as cpFloat, d as cpFloat ) as cpFloat
	return f1 + cpfclamp( f2 - f1, -d, d )
end function

type cpVect
	as cpFloat	x, y
end type

type cpHashValue		as unsigned integer

'' Oh C, how we love to define our own boolean types to get compiler compatibility
#ifdef CP_BOOL_TYPE
	type cpBool as CP_BOOL_TYPE
#else
	type cpBool as integer
#endif

#ifndef cpTrue
#	define cpTrue 1
#endif

#ifndef cpFalse
#	define cpFalse 0
#endif
type cpDataPointer		as any ptr
type cpCollisionType	as unsigned integer
type cpGroup			as unsigned integer
type cpLayers			as unsigned integer

#ifdef CP_TIMESTAMP_TYPE
	type cpTimestamp as CP_TIMESTAMP_TYPE
#else
	type cpTimestamp as unsigned integer
#endif

#ifndef CP_NO_GROUP
#	define CP_NO_GROUP (cast(cpGroup, 0))
#endif

#ifndef CP_ALL_LAYERS
#	define CP_ALL_LAYERS (!cast(cpLayers, 0))
#endif
