/' Copyright (c) 2007 Scott Lembcke
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 '/

'' The spatial hash is Chipmunk's default (and currently only) spatial index type.
'' Based on a chained hash table.

'' Used internally to track objects added to the hash
type cpHandle
    '' Pointer to the object
    as any ptr		obj
    '' Retain count
    as integer		retain
    '' Query stamp. Used to make sure two objects
    '' aren't identified twice in the same query.
    as cpTimestamp	stamp
end type

'' Linked list element for in the chains.
type cpSpaceHashBin
	as cpHandle ptr			handle
	as cpSpaceHashBin ptr	next
end type

'' BBox callback. Called whenever the hash needs a bounding box from an object.
type cpSpaceHashBBFunc as function cdecl( byval as any ptr ) as cpBB

type cpSpaceHash
	'' Number of cells in the table.
	as integer		numcells
	'' Dimentions of the cells.
	as cpFloat		celldim
	
	'' BBox callback.
	as cpSpaceHashBBFunc	bbfunc

	'' Hashset of the handles and the recycled ones.
	as cpHashSet ptr	handleSet
	as cpArray ptr		pooledHandles
	
	'' The table and the recycled bins.
	as cpSpaceHashBin ptr ptr	table
	as cpSpaceHashBin ptr		pooledBins
	
	'' list of buffers to free on destruction.
	as cpArray ptr		allocatedBuffers
	
	'' Incremented on each query. See cpHandle.stamp.
	as cpTimestamp		stamp
end type

'' Basic allocation/destruction functions.
declare function cpSpaceHashAlloc cdecl alias "cpSpaceHashAlloc" ( ) as cpSpaceHash ptr
declare function cpSpaceHashInit cdecl alias "cpSpaceHashInit" ( byval as cpSpaceHash ptr, byval as cpFloat, byval as integer, byval as cpSpaceHashBBFunc ) as cpSpaceHash ptr
declare function cpSpaceHashNew cdecl alias "cpSpaceHashNew" ( byval as cpFloat, byval as integer, byval as cpSpaceHashBBFunc ) as cpSpaceHash ptr

declare sub cpSpaceHashDestroy cdecl alias "cpSpaceHashDestroy" ( byval as cpSpaceHash ptr )
declare sub cpSpaceHashFree cdecl alias "cpSpaceHashFree" ( byval as cpSpaceHash ptr )

'' Resize the hashtable. (Does not rehash! You must call cpSpaceHashRehash() if needed.)
declare sub cpSpaceHashResize cdecl alias "cpSpaceHashResize" ( byval as cpSpaceHash ptr, byval as cpFloat, byval as integer )

'' Add an object to the hash.
declare sub cpSpaceHashInsert cdecl alias "cpSpaceHashInsert" ( byval as cpSpaceHash ptr, byval as any ptr, byval as cpHashValue, byval as cpBB )
'' Remove an object from the hash.
declare sub cpSpaceHashRemove cdecl alias "cpSpaceHashRemove" ( byval as cpSpaceHash ptr, byval as any ptr, byval as cpHashValue )

'' Iterator function
type cpSpaceHashIterator as sub cdecl( byval as any ptr, byval as any ptr )
'' Iterate over the objects in the hash.
declare sub cpSpaceHashEach cdecl alias "cpSpaceHashEach" ( byval as cpSpaceHash ptr, byval as cpSpaceHashIterator, byval as any ptr )

'' Rehash the contents of the hash.
declare sub cpSpacehashRehash cdecl alias "cpSpaceHashRehash" ( byval as cpSpaceHash ptr )
'' Rehash only a specific object.
declare sub cpSpaceHashRehashObject cdecl alias "cpSpacehashRehashObject" ( byval as cpSpaceHash ptr, byval as any ptr, byval as unsigned integer )

'' Query callback.
type cpSpaceHashQueryFunc as sub cdecl( byal as any ptr, byval as any ptr, byval as any ptr )
'' Query the hash for a given BBox.
declare sub cpSpaceHashQuery cdecl alias "cpSpaceHashQuery" ( byval as cpSpaceHash ptr, byval as any ptr, byval as cpBB, byval as cpSpaceHashQueryFunc, byval as any ptr )
'' Run a query for the object, then insert it. (Optimized case)
declare sub cpSpaceHashQueryInsert cdecl alias "cpSpaceHashQueryInsert" ( byval as cpSpaceHash ptr, byval as any ptr, byval as cpBB, byval as cpSpaceHashQueryFunc, byval as any ptr )
'' Rehashes while querying for each object. (Optimized case) 
declare sub cpSpaceHashQueryRehash cdecl alias "cpSpaceHashQueryRehash" ( byval as cpSpaceHash ptr, byval as cpSpaceHashQueryFunc, byval as any ptr )

'' Segment Query callback.
'' Return value is uesd for early exits of the query.
'' If while traversing the grid, the raytrace function detects that an entire grid cell is beyond the hit point, it will stop the trace.
type cpSpaceHashSegmentQueryFunc as function cdecl( byval as any ptr, byval as any ptr, byval as any ptr ) as cpFloat
declare sub cpSpaceHashSegmentQuery cdecl alias "cpSpaceHashSegmentQuery" ( byval as cpSpaceHash ptr, byval as any ptr, byval as cpVect, byval as cpVect, byval as cpFloat, byval as cpSpaceHashSegmentQueryFunc, byval as any ptr )

